# -*- coding: utf8 -*-
from __future__ import unicode_literals

import logging

from driverng import Worklist
from driverng.fixed import FixedLengthFormat, Multiple, String
from driverng.drivers.au.base import (
    AuOrder,
    AuQuery,
    AuResult,
    AuRerunOrder,
    AuRerunResult,
    AuQC,
    AuBaseDriver,
)

logger = logging.getLogger("driverng.dxc700")


class DxC700ISEResult(FixedLengthFormat):
    TEST = [
        String("test_number", 3),
        String("analysis_data", 9),
        String("data_flags", 8),
    ]
    FMT_DESCRIPTION = (
        [String("code", 2), String("unit_no", 2)]
        + AuOrder.HEADER
        + AuOrder.PATIENT
        + [Multiple("tests", TEST)]
    )


class DxC700ISERerunResult(FixedLengthFormat):
    FMT_DESCRIPTION = (
        [String("code", 2), String("unit_no", 2)]
        + AuOrder.HEADER
        + [Multiple("tests", DxC700ISEResult.TEST)]
    )


class DxC700ISEQC(FixedLengthFormat):
    FMT_DESCRIPTION = (
        [String("code", 2), String("unit_no", 2)]
        + AuOrder.HEADER
        + [Multiple("tests", DxC700ISEResult.TEST)]
    )


class DxC700Driver(AuBaseDriver):
    SPEC = {"code": "dxc700", "label": "DxC 700 AU"}

    @staticmethod
    def get_klass(code, test_number):
        if code in ("Sh", "SH"):
            return AuRerunOrder
        elif code[0] == "S":
            return AuOrder
        elif code[0] == "R":
            return AuQuery
        elif code in ("DQ", "DR", "DA") and test_number not in ("097", "098", "099"):
            return AuQC
        elif code in ("DQ", "DR", "DA") and test_number in ("097", "098", "099"):
            return DxC700ISEQC
        elif code in ("Dh", "DH") and test_number not in ("097", "098", "099"):
            return AuRerunResult
        elif code in ("Dh", "DH") and test_number in ("097", "098", "099"):
            return DxC700ISERerunResult
        elif code[0] == "D" and test_number not in ("097", "098", "099"):
            return AuResult
        elif code[0] == "D" and test_number in ("097", "098", "099"):
            return DxC700ISEResult
        else:
            raise NotImplementedError

    def create(self, code, **kwargs):
        kwargs.setdefault("code", code)
        test_number = kwargs.setdefault("test_number")
        return self.get_klass(code, test_number)(**kwargs)

    def parse(self, data):
        test_number = data[82:85]
        if data[:2] in ("DQ", "DR", "DA"):
            test_number = data[36:39]
        obj = self.get_klass(data[:2], test_number)()
        obj.parse(data)
        return obj

    def decode_message(self, message):
        if isinstance(message, AuQuery):
            return self.get_query_worklist(message)

        worklist = Worklist(context="result")
        is_ise = False
        if isinstance(message, AuResult) or isinstance(message, AuRerunResult):
            sample = self.get_result_worlist(message, worklist)
        elif isinstance(message, DxC700ISEResult) or isinstance(
            message, DxC700ISERerunResult
        ):
            sample = self.get_result_worlist(message, worklist)
            is_ise = True
        elif isinstance(message, AuQC):
            sample = self.get_qc_worklist(message, worklist)
        elif isinstance(message, DxC700ISEQC):
            sample = self.get_qc_worklist(message, worklist)
            is_ise = True
        else:
            raise NotImplementedError

        for test in message.tests:
            flags = []
            if test.data_flags:
                for flag in test.data_flags:
                    if flag.strip():
                        flags.append(flag)
            lots = []
            if not is_ise:
                reagents = self.get_reagents(test.reagent)
                for reagent in reagents:
                    lots.append({"lot": reagent, "type": "reagent"})
            sample.results.add(
                code=test.test_number, value=test.analysis_data, flags=flags, lots=lots
            )
            if test.test_number == "096":
                splitted = test.analysis_data.split()
                if len(splitted) == 3:
                    for code, value in zip(["L", "I", "H"], splitted):
                        sample.results.add(
                            code=code, value=value, flags=flags, lots=lots
                        )

        return worklist
