# -*- coding: utf8 -*-
from __future__ import unicode_literals

from unidecode import unidecode

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    HprimDate,
    HprimForeign,
    HprimList,
    HprimMultiple,
    HprimString,
    Segment,
)


class UwamSegmentH(HeaderSegment):

    sender_name = HprimString((5, 1))
    software_version = HprimString((5, 2))
    serial_no = HprimString((5, 3))
    ps_code = HprimString((5, 4))
    version = HprimString(13, default="LIS2-A2")

    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    scientifics = HprimForeign("S", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class UwamSegmentP(Segment):
    patient_id = HprimString(3)
    laboratory_patient_id = HprimString(4, max_length=16)
    last_name = HprimString((6, 1), max_length=20)
    first_name = HprimString((6, 2), max_length=20)
    birthdate = HprimDate(8, fmt_date="%Y%m%d")
    # M/F/U
    sex = HprimString(9)
    special_field1 = HprimString(15)
    admission_status = HprimString(26)

    orders = HprimForeign("O", "patient")


class UwamSegmentO(Segment):
    sample_number = HprimString((3, 1))
    rack_number = HprimString((3, 2))
    rack_position = HprimString((3, 3))
    test_id = HprimList(5, basefield=HprimMultiple(0, parameter_id=HprimString(4)))
    # R: Routine
    # A: Urgent
    # S: Emergency
    priority = HprimString(6)
    collection_date = HprimDate(8)
    # C: Cancel
    # A: Add
    # N: New
    # Q: QC
    action_code = HprimString(12)
    date_specimen_received = HprimDate(15)
    # Urine/Urine-EarlyMorning/Urine-Pooled/Urine-Postprandial/Urine-Catheter/Blank
    specimen_descriptor = HprimString(16)
    physician_code = HprimString((17, 1))
    physician_last_name = HprimString((17, 2))
    physician_first_name = HprimString((17, 3))
    physician_title = HprimString((17, 4))
    physician_phone = HprimString(18)
    order_no = HprimString(19)
    report_type = HprimString(26, default="O")
    # Location fields or Ward fields
    ward_code = HprimString((28, 1))
    ward_name = HprimString((28, 2))
    location_phone = HprimString((28, 3))
    specimen_descriptor = HprimString(16)
    medical_section_code = HprimString((31, 1))
    medical_section_name = HprimString((31, 2))

    results = HprimForeign("R", "order")


class UwamSegmentR(Segment):
    test_id = HprimString((3, 4))
    analysis_method = HprimString((3, 5))
    dilution = HprimString((3, 6))
    result_type = HprimString((3, 7))
    rack_no = HprimString((3, 8))
    rack_position = HprimString((3, 9))
    value = HprimString((4, 1))
    format_value = HprimString((4, 2))
    units = HprimString(5)
    reference_range = HprimString(6)
    # A, N, H, >, W, HH
    abnormal_flag = HprimString(7)
    operator_user_id = HprimString((11, 3))
    operator_user_name = HprimString((11, 4))
    date_result_completed = HprimDate(13)
    instrument_id = HprimString(14)


class UwamSegmentQ(Segment):
    sample_number = HprimString(3)
    starting_date_request = HprimDate(7)


class UwamSegmentC(Segment):
    # Indicates the comment type
    # (None): Report comment
    # P: Patient comment
    # S: Sample comment
    comment_text_type = HprimString((4, 1))
    comment = HprimString((4, 2))


class UwamSegmentS(Segment):
    qc_lot = HprimString(7)


class UwamSegmentL(Segment):
    terminaison_code = HprimString(3, default="N")


class Driver(AstmMessageV2):
    SPEC = {"code": "uwam", "label": "U-WAM SYSMEX"}
    SEGMENT_CLASS_MAP = {
        "H": UwamSegmentH,
        "P": UwamSegmentP,
        "Q": UwamSegmentQ,
        "O": UwamSegmentO,
        "R": UwamSegmentR,
        "S": UwamSegmentS,
        "C": UwamSegmentC,
        "L": UwamSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    OPTIONS = [
        AstmMessageV2.Option(
            "uwam_cristaux",
            label="Liste des cristaux",
            default="UAMOR;PAMOR;OXCAL;ACUR;PHOSCA;PAM;URAMO;CARCA;BILI;CYST;CHOL;TYRO;LEUC",
        ),
        AstmMessageV2.Option(
            "uwam_cylindres",
            label="Liste des cylindres",
            default="CYLEP;CYLGRA;CYLHYA;CYLRBC;CYLWBC",
        ),
    ]

    def __init__(
        self,
        uwam_cristaux="UAMOR;PAMOR;OXCAL;ACUR;PHOSCA;PAM;URAMO;CARCA;BILI;CYST;CHOL;TYRO;LEUC",
        uwam_cylindres="CYLEP;CYLGRA;CYLHYA;CYLRBC;CYLWBC",
        **kwargs
    ):
        self.cristaux = uwam_cristaux
        self.cylindres = uwam_cylindres
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        message = UwamSegmentH(type_segment="H", **kwargs)
        message.end = UwamSegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        if worklist.context != "order":
            raise NotImplementedError("context {}".format(worklist.context))

        msg = self.create()
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            action={"del": "C", "add": "A", "new": "N"},
            priority={"A": "A", "S": "S", worklist.ELSE: "R"},
        ):
            seg_p = msg.patients.add(
                patient_id=patient.patient_id,
                laboratory_patient_id=patient.lab_patient_id,
                last_name=patient.last_name,
                first_name=patient.first_name,
                birthdate=patient.birthdate,
                sex=patient.sex,
            )
            for sample, action, priority in patient.samples:
                seg_o = seg_p.orders.add(
                    sample_number=sample.sample_id,
                    priority=priority,
                    collection_date=sample.collection_date,
                    action_code=action,
                    specimen_descriptor=sample.specimen,
                )
                for physician in sample.physicians:
                    seg_o.physician_code = "{:.20s}".format(unidecode(physician.code))
                    seg_o.physician_last_name = (
                        "{:.20s}".format(physician.last_name)
                        if physician.last_name
                        else ""
                    )
                    seg_o.physician_first_name = (
                        "{:.20s}".format(physician.first_name)
                        if physician.first_name
                        else ""
                    )
                for order in sample.orders:
                    test = seg_o.test_id.add()
                    test.parameter_id = order.code
        return msg

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                worklist.get_or_create(type="patient", sample_id=seg_q.sample_number)
        else:
            worklist = Worklist(context="result")
            for seg_p in message.patients:
                for seg_o in seg_p.orders:
                    if seg_o.action_code == "Q":
                        lot = ""
                        if seg_o.sample_number.startswith("QC_"):
                            lot = seg_o.sample_number[3:]
                        sample = worklist.get_or_create(
                            sample_id=lot if lot else seg_o.sample_number,
                            control_lot=lot if lot else seg_o.sample_number,
                            type="control",
                        )
                    else:
                        if not seg_o.rack_number and not seg_o.rack_position:
                            rack = ""
                        else:
                            rack = "{}-{}".format(
                                seg_o.rack_number, seg_o.rack_position
                            )
                        sample = worklist.get_or_create(
                            sample_id=seg_o.sample_number,
                            specimen=seg_o.specimen_descriptor,
                            rack=rack,
                            type="patient",
                        )
                    comments = []
                    cristaux = []
                    cylindres = []
                    instrument = None
                    for seg_r in seg_o.results:
                        instrument = seg_r.instrument_id
                        if seg_r.value == "":
                            continue
                        for seg_c in seg_r.commentaires:
                            comments.extend(seg_c.comment)
                        if (
                            seg_r.test_id in self.cristaux.split(";")
                            and seg_r.format_value == "MAINFORMAT"
                        ):
                            resu_cristal = "{}{}".format(seg_r.test_id, seg_r.value)
                            cristaux.append(resu_cristal)
                        elif (
                            seg_r.test_id in self.cylindres.split(";")
                            and seg_r.format_value == "MAINFORMAT"
                        ):
                            resu_cylindre = "{}{}".format(seg_r.test_id, seg_r.value)
                            cylindres.append(resu_cylindre)
                        elif (
                            seg_r.test_id in self.cristaux.split(";")
                            and seg_r.format_value == "RAW"
                            or seg_r.test_id in self.cylindres.split(";")
                            and seg_r.format_value == "RAW"
                        ):
                            continue
                        else:
                            code = "{}{}".format(seg_r.test_id, seg_r.format_value)
                            value = seg_r.value
                            # Images
                            if seg_r.result_type == "IF":
                                code = seg_r.test_id
                                values = seg_r.value.split("\\")
                                for v in values:
                                    if ".png" in v:
                                        value = v
                            sample.results.add(
                                code=code,
                                value=value,
                                flags=[seg_r.abnormal_flag],
                                comments=comments,
                                dilution=seg_r.dilution,
                                operator=seg_r.operator_user_id,
                                instrument=seg_r.instrument_id,
                                date=seg_r.date_result_completed,
                            )
                    # Traitement des cristaux et des cylindres
                    if cristaux:
                        sample.results.add(
                            code="CRISTAUX",
                            value=";".join(cristaux),
                            instrument=instrument,
                        )
                    else:
                        sample.results.add(
                            code="CRISTAUX", value="ABS", instrument=instrument
                        )
                    if cylindres:
                        sample.results.add(
                            code="CYLINDRES",
                            value=";".join(cylindres),
                            instrument=instrument,
                        )
                    else:
                        sample.results.add(
                            code="CYLINDRES", value="ABS", instrument=instrument
                        )
        return worklist
